import click
from werkzeug.utils import import_string


def get_generator(name):
    if '.' not in name:
        name = 'flask_board.generator.{}Generator'.format(name.capitalize())
    try:
        cls = import_string(name)
        return cls
    except ImportError:
        click.echo('No generator {} found!'.format(name))


@click.command(short_help='Generate flask app by templates')
@click.argument('name')
@click.option('-d', '--directory', help='Project directory, default current directory')
@click.option('-g', '--generator', help='Generator type', type=str, default='jinja2', show_default=True)
@click.option('-t', '--template', help='Template name', default='default')
@click.option('-p', '--template-dir', help='Use custom template directory')
@click.option('-a', '--additional', help='Additional parameters send to generator', multiple=True)
@click.option('-s', '--skip', help='Skip prompts and use default', is_flag=True)
def cli(name, directory, generator, template, **kwargs):
    generator = get_generator(generator)(name=name, directory=directory, template=template)
    try:
        generator.run(**kwargs)
    except (FileNotFoundError, FileExistsError) as e:
        click.echo(e)



import os
from werkzeug.utils import import_string
from logging.config import dictConfig
from flask import Flask


# Environment key to config flask instance path
ENV_INSTANCE_PATH = 'FLASK_INSTANCE_PATH'
# Environment key to config flask config module
ENV_CONFIG_MODULE = 'FLASK_CONFIG_MODULE'


def config_logger(app):
    if 'LOGGER' in app.config:
        dictConfig(app.config['LOGGER'])


def register_app(app):
    """
    Import blueprints module in REGISTERED_APP config.

    :param app: flask app
    """
    with app.app_context():
        for a in app.config['REGISTERED_APP']:
            bp = '{}.blueprints'.format(a)
            import_string(bp)


def create_app_by_config(conf=None):
    """
    Create flask app by config object.

    :param conf: config object
    :return: flask app
    """
    # check instance path
    instance_path = os.environ.get(ENV_INSTANCE_PATH) or None
    # create app
    app = Flask(__name__, instance_path=instance_path)
    # ensure the instance folder exists
    if app.instance_path:
        try:
            os.makedirs(app.instance_path, exist_ok=True)
        except OSError:
            pass
    # configure app
    if conf:
        app.config.from_object(conf)
    # config logger
    config_logger(app)
    # register app
    register_app(app)
    return app


def create_app():
    """
    Create flask app by config module.

    :return: flask app
    """
    config = os.environ.get(ENV_CONFIG_MODULE) or 'config'
    return create_app_by_config(config)



import os
import shutil
import random
import string
import click


def generate_random_str(length=16):
    return ''.join(random.choices(string.ascii_letters + string.digits, k=length))


class AppGenerator:

    def __init__(self, name, directory, template):
        self.name = name
        self.directory = directory or ''
        self.template = template
        self.skip_prompt = False

    @property
    def app_dir(self):
        if self.directory:
            path = os.path.join(self.directory, self.name)
        else:
            path = self.name
        return path

    def create_app_dir(self, exist_ok=False):
        """
        Create app directory.

        :param bool exist_ok:
        """
        path = self.app_dir
        if not exist_ok and os.path.exists(path):
            raise FileExistsError('App path {} already exists!'.format(path))
        os.makedirs(path, exist_ok=exist_ok)

    def run(self, **kwargs):
        """
        Run generator.
        """
        kwargs = self.pre_process(**kwargs)
        self.generate(**kwargs)
        self.post_process(**kwargs)

    def pre_process(self, **kwargs):
        """
        Pre process before generation.
        """
        if 'skip' in kwargs and kwargs['skip'] is True:
            self.skip_prompt = True
        self.create_app_dir()
        return kwargs

    def generate(self, **kwargs):
        """
        Generate project from template.
        """
        yield NotImplementedError

    def post_process(self, **kwargs):
        """
        Post process after generation
        """
        pass


class FileGenerator(AppGenerator):

    category = 'file'

    @classmethod
    def get_internal_template_dir(cls):
        return os.path.realpath(os.path.join(os.path.dirname(__file__), 'templates', cls.category))

    @staticmethod
    def get_rel_path(base_path, path):
        if base_path == path:
            return ''
        elif len(base_path) < len(path):
            return path[len(base_path) + 1:]
        else:
            raise ValueError('Path {} must larger than base path {}!'.format(path, base_path))

    def find_template(self, template_dir=None):
        if not template_dir:
            template_dir = self.get_internal_template_dir()
        path = os.path.join(template_dir, self.template)
        if os.path.exists(path):
            return path
        raise FileNotFoundError('Template path {} not found!'.format(path))

    def pre_process(self, **kwargs):
        kwargs = super().pre_process(**kwargs)
        template_dir = kwargs.get('template_dir')
        path = self.find_template(template_dir=template_dir)
        kwargs['template_path'] = path
        return kwargs

    def generate(self, **kwargs):
        path = kwargs.get('template_path')
        for name, dirs, files in os.walk(path):
            rel_path = self.get_rel_path(path, name)
            # create sub directories
            for d in dirs:
                os.makedirs(os.path.join(self.app_dir, rel_path, d), exist_ok=True)
            # get templates
            if rel_path:
                tmpls = [os.path.join(rel_path, t) for t in files]
            else:
                tmpls = files
            for tmpl in tmpls:
                shutil.copyfile(os.path.join(path, tmpl), os.path.join(self.app_dir, tmpl))


class Jinja2Generator(FileGenerator):

    category = 'jinja2'
    index_list = [
        'https://pypi.org/simple',
        'https://pypi.tuna.tsinghua.edu.cn/simple/',
        'https://mirrors.aliyun.com/pypi/simple',
        'https://pypi.doubanio.com/simple/',
    ]

    def get_context(self, **kwargs):
        context = {
            'name': self.name,
            'directory': self.directory,
            'template': self.template
        }
        context.update(kwargs)
        return context

    def pre_process(self, **kwargs):
        kwargs = super().pre_process(**kwargs)
        # generate render params
        additional = kwargs.get('additional')
        params = {'secret': generate_random_str()}
        if additional:
            for a in additional:
                idx = a.find('=')
                if idx > 0:
                    params[a[0:idx]] = a[idx + 1:]
        # select package index
        if 'pip_index' not in params:
            pip_index = 1
            if not self.skip_prompt:
                pip_index = 0
                msg = ['Select which Python Package Index to use: ', '']
                msg += ['[{}] {}'.format(i + 1, self.index_list[i]) for i in range(len(self.index_list))]
                msg += ['', 'Please input the front number: ']
                while pip_index <= 0 or pip_index > len(self.index_list):
                    pip_index = click.prompt('\n'.join(msg), type=int, default=1)
            params['pip_index'] = self.index_list[pip_index - 1]
        # set render_params
        kwargs['render_params'] = params
        return kwargs

    def generate(self, **kwargs):
        path = kwargs.get('template_path')
        params = kwargs.get('render_params') or {}
        context = self.get_context(template_path=path)
        # load templates
        from jinja2 import Environment, FileSystemLoader
        env = Environment(loader=FileSystemLoader(path))
        for name, dirs, files in os.walk(path):
            rel_path = self.get_rel_path(path, name)
            # create sub directories
            for d in dirs:
                os.makedirs(os.path.join(self.app_dir, rel_path, d), exist_ok=True)
            # get templates
            if rel_path:
                tmpls = [os.path.join(rel_path, t) for t in files if not t.endswith('.pyc')]
            else:
                tmpls = files
            for tmpl in tmpls:
                print('-----', tmpl)
                template = env.get_template(tmpl, globals={'context': context})
                target = os.path.join(self.app_dir, tmpl)
                with open(target, 'w') as fh:
                    fh.write(template.render(**params))



from logging.config import dictConfig


DEFAULT_FORMATTER = '[%(asctime)s] %(levelname)s in %(module)s: %(message)s'
DETAIL_FORMATTER = '[%(asctime)s] %(levelname)s in %(module)s: %(message)s'


def config_logger(enable_console_handler=True, enable_file_handler=True, log_file='app.log', log_level='ERROR',
                  log_file_max_bytes=5000000, log_file_max_count=5, default_formatter=None, detail_formatter=None):
    console_handler = {
        'class': 'logging.StreamHandler',
        'formatter': 'default',
        'level': log_level,
        'stream': 'ext://flask.logging.wsgi_errors_stream'
    }
    file_handler = {
        'class': 'logging.handlers.RotatingFileHandler',
        'formatter': 'detail',
        'filename': log_file,
        'level': log_level,
        'maxBytes': log_file_max_bytes,
        'backupCount': log_file_max_count
    }
    default_formatter = {
        'format': default_formatter or DEFAULT_FORMATTER
    }
    detail_formatter = {
        'format': detail_formatter or DETAIL_FORMATTER
    }
    handlers = []
    if enable_console_handler:
        handlers.append('console')
    if enable_file_handler:
        handlers.append('file')
    d = {
        'version': 1,
        'formatters': {
            'default': default_formatter,
            'detail': detail_formatter
        },
        'handlers': {
            'console': console_handler,
            'file': file_handler
        },
        'root': {
            'level': log_level,
            'handlers': handlers
        }
    }
    dictConfig(d)
